@@ -1,4 +1,4 @@
-# Copyright 2014 Jeffrey Kegler
+# Copyright 2015 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -20,7 +20,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.102000';
+$VERSION        = '2.104000';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -213,18 +213,75 @@ sub Marpa::R2::Scanless::R::new {
     $slr->[Marpa::R2::Internal::Scanless::R::TRACE_LEXERS] = 0;
     $slr->[Marpa::R2::Internal::Scanless::R::TRACE_TERMINALS] = 0;
 
-    my $g1_recce_args =
+    my ($g1_recce_args, $flat_args) =
         Marpa::R2::Internal::Scanless::R::set( $slr, "new", @args );
     my $too_many_earley_items = $g1_recce_args->{too_many_earley_items};
 
     my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
 
+    Marpa::R2::exception(
+        qq{Marpa::R2::Scanless::R::new() called without a "grammar" argument}
+    ) if not defined $slg;
+
+    my $slg_class = 'Marpa::R2::Scanless::G';
+    if ( not blessed $slg or not $slg->isa($slg_class) ) {
+        my $ref_type = ref $slg;
+        my $desc = $ref_type ? "a ref to $ref_type" : 'not a ref';
+        Marpa::R2::exception(
+            qq{'grammar' named argument to new() is $desc\n},
+            "  It should be a ref to $slg_class\n" );
+    } ## end if ( not blessed $slg or not $slg->isa($slg_class) )
+
     my $thick_g1_grammar =
         $slg->[Marpa::R2::Internal::Scanless::G::THICK_G1_GRAMMAR];
-    $g1_recce_args->{grammar} = $thick_g1_grammar;
+
+    my $trace_file_handle = $g1_recce_args->{trace_file_handle};
+    $trace_file_handle //= $thick_g1_grammar->[Marpa::R2::Internal::Grammar::TRACE_FILE_HANDLE] ;
+
     my $thick_g1_recce =
-        $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE] =
-        Marpa::R2::Recognizer->new($g1_recce_args);
+        $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE] = bless [],
+        'Marpa::R2::Recognizer';
+
+    local $Marpa::R2::Internal::TRACE_FH =
+        $thick_g1_recce->[Marpa::R2::Internal::Recognizer::TRACE_FILE_HANDLE] = $trace_file_handle;
+
+    $thick_g1_recce->[Marpa::R2::Internal::Recognizer::GRAMMAR] = $thick_g1_grammar;
+
+    my $grammar_c = $thick_g1_grammar->[Marpa::R2::Internal::Grammar::C];
+
+    my $recce_c = $thick_g1_recce->[Marpa::R2::Internal::Recognizer::C] =
+        Marpa::R2::Thin::R->new($grammar_c);
+    if ( not defined $recce_c ) {
+        Marpa::R2::exception( $grammar_c->error() );
+    }
+
+    $recce_c->ruby_slippers_set(1);
+
+    if (   defined $thick_g1_grammar->[Marpa::R2::Internal::Grammar::ACTION_OBJECT]
+        or defined $thick_g1_grammar->[Marpa::R2::Internal::Grammar::ACTIONS]
+        or not defined $thick_g1_grammar->[Marpa::R2::Internal::Grammar::INTERNAL] )
+    {
+        $thick_g1_recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE_SOURCE] =
+            'legacy';
+    } ## end if ( defined $grammar->[...])
+
+    if ( defined( my $value = $g1_recce_args->{'leo'} ) ) {
+            my $boolean = $value ? 1 : 0;
+            $thick_g1_recce->use_leo_set($boolean);
+            delete $g1_recce_args->{leo};
+        }
+
+    $thick_g1_recce->[Marpa::R2::Internal::Recognizer::WARNINGS]       = 1;
+    $thick_g1_recce->[Marpa::R2::Internal::Recognizer::RANKING_METHOD] = 'none';
+    $thick_g1_recce->[Marpa::R2::Internal::Recognizer::MAX_PARSES]     = 0;
+    $thick_g1_recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS]     = 0;
+
+    # Position 0 is not used because 0 indicates an unvalued token.
+    # Position 1 is reserved for undef.
+    # Position 2 is reserved for literal tokens (used in SLIF).
+    $thick_g1_recce->[Marpa::R2::Internal::Recognizer::TOKEN_VALUES] = [undef, undef, undef];
+
+    $thick_g1_recce->reset_evaluation();
 
     my $thin_slr =
         Marpa::R2::Thin::SLR->new( $slg->[Marpa::R2::Internal::Scanless::G::C],
@@ -233,6 +290,68 @@ sub Marpa::R2::Scanless::R::new {
         if defined $too_many_earley_items;
     $slr->[Marpa::R2::Internal::Scanless::R::C]      = $thin_slr;
     $slr->[Marpa::R2::Internal::Scanless::R::EVENTS] = [];
+
+    my $symbol_ids_by_event_name_and_type =
+        $slg->[
+        Marpa::R2::Internal::Scanless::G::SYMBOL_IDS_BY_EVENT_NAME_AND_TYPE];
+
+    my $event_is_active_arg = $flat_args->{event_is_active} // {};
+    if (ref $event_is_active_arg ne 'HASH') {
+        Marpa::R2::exception( 'event_is_active named argument must be ref to hash' );
+    }
+
+    # Completion/nulled/prediction events are always initialized by
+    # Libmarpa to 'on'.  So here we need to override that if and only
+    # if we in fact want to initialize them to 'off'.
+
+    # Events are already initialized as described by
+    # the DSL.  Here we override that with the recce arg, if
+    # necessary.
+    
+    EVENT: for my $event_name ( keys %{$event_is_active_arg} ) {
+
+        my $is_active = $event_is_active_arg->{$event_name};
+
+        my $symbol_ids =
+            $symbol_ids_by_event_name_and_type->{$event_name}->{lexeme};
+        $thin_slr->lexeme_event_activate( $_, $is_active )
+            for @{$symbol_ids};
+        my $lexer_rule_ids =
+            $symbol_ids_by_event_name_and_type->{$event_name}->{discard};
+        $thin_slr->discard_event_activate( $_, $is_active )
+            for @{$lexer_rule_ids};
+
+        $symbol_ids =
+            $symbol_ids_by_event_name_and_type->{$event_name}->{completion}
+            // [];
+        $recce_c->completion_symbol_activate( $_, $is_active )
+            for @{$symbol_ids};
+        $symbol_ids =
+            $symbol_ids_by_event_name_and_type->{$event_name}->{nulled} // [];
+        $recce_c->nulled_symbol_activate( $_, $is_active ) for @{$symbol_ids};
+        $symbol_ids =
+            $symbol_ids_by_event_name_and_type->{$event_name}->{prediction}
+            // [];
+        $recce_c->prediction_symbol_activate( $_, $is_active )
+            for @{$symbol_ids};
+    } ## end EVENT: for my $event_name ( keys %{$event_is_active_arg} )
+
+    if ( not $recce_c->start_input() ) {
+        my $error = $grammar_c->error();
+        Marpa::R2::exception( 'Recognizer start of input failed: ', $error );
+    }
+
+    $thick_g1_recce->set($g1_recce_args);
+
+    if ( $thick_g1_recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] > 1 ) {
+        my @terminals_expected = @{ $thick_g1_recce->terminals_expected() };
+        for my $terminal ( sort @terminals_expected ) {
+            say {$Marpa::R2::Internal::TRACE_FH}
+                qq{Expecting "$terminal" at earleme 0}
+                or Marpa::R2::exception("Cannot print: $ERRNO");
+        }
+    } ## end if ( $thick_g1_recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS...])
+
     Marpa::R2::Internal::Scanless::convert_libmarpa_events($slr);
 
     return $slr;
@@ -273,7 +392,7 @@ sub Marpa::R2::Internal::Scanless::R::set {
         )
     };
     state $new_method_args = {
-        map { ( $_, 1 ); } qw(grammar ranking_method),
+        map { ( $_, 1 ); } qw(grammar ranking_method event_is_active),
         keys %{$set_method_args}
     };
     state $series_restart_method_args = {
@@ -319,26 +438,6 @@ sub Marpa::R2::Internal::Scanless::R::set {
         );
     } ## end if ( scalar @bad_args )
 
-    if ( $method eq 'new' ) {
-        state $arg_name  = 'grammar';
-        state $slg_class = 'Marpa::R2::Scanless::G';
-        my $slg = $flat_args{$arg_name};
-        Marpa::R2::exception(
-            qq{Marpa::R2::Scanless::R::new() called without a "$arg_name" argument}
-        ) if not defined $slg;
-        if ( not blessed $slg or not $slg->isa($slg_class) ) {
-            my $ref_type = ref $slg;
-            my $desc = $ref_type ? "a ref to $ref_type" : 'not a ref';
-            Marpa::R2::exception(
-                qq{'$arg_name' name argument to scanless_r->new() is $desc\n},
-                "  It should be a ref to $slg_class\n"
-            );
-        } ## end if ( not blessed $slg or not $slg->isa($slg_class) )
-
-        $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR] = $slg;
-
-    } ## end if ( $method eq 'new' )
-
     # Special SLIF (not NAIF) recce arg processing goes here
     if ( exists $flat_args{'exhaustion'} ) {
 
@@ -375,9 +474,11 @@ sub Marpa::R2::Internal::Scanless::R::set {
     # the Scanless::R class, so this maps named args to the index of the array
     # that holds the members.
     state $copy_arg_to_index = {
-        trace_file_handle => Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE,
-        trace_lexers      => Marpa::R2::Internal::Scanless::R::TRACE_LEXERS,
-        trace_terminals   => Marpa::R2::Internal::Scanless::R::TRACE_TERMINALS,
+        trace_file_handle =>
+            Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE,
+        trace_lexers    => Marpa::R2::Internal::Scanless::R::TRACE_LEXERS,
+        trace_terminals => Marpa::R2::Internal::Scanless::R::TRACE_TERMINALS,
+        grammar         => Marpa::R2::Internal::Scanless::R::GRAMMAR,
     };
 
     ARG: for my $arg_name ( keys %flat_args ) {
@@ -416,18 +517,14 @@ sub Marpa::R2::Internal::Scanless::R::set {
 
     # Prune flat args of all those named args which are NOT to be copied
     # into the NAIF recce args
-    for my $arg_name ( keys %flat_args ) {
-        delete $flat_args{$arg_name}
-            if not $copyable_naif_recce_args->{$arg_name};
+    my %g1_recce_args = ();
+    for my $arg_name ( grep { $copyable_naif_recce_args->{$_} }
+        keys %flat_args )
+    {
+        $g1_recce_args{$arg_name} = $flat_args{$arg_name};
     }
 
-    # In the new method, these items must always be set in the NAIF recce args
-    if ( $method eq 'new' ) {
-        $flat_args{trace_file_handle} //=
-            $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
-    }
-
-    return \%flat_args;
+    return \%g1_recce_args, \%flat_args;
 
 } ## end sub Marpa::R2::Internal::Scanless::R::set
 
@@ -486,21 +583,11 @@ sub Marpa::R2::Scanless::R::read {
 
 } ## end sub Marpa::R2::Scanless::R::read
 
-sub Marpa::R2::Scanless::R::lexer_set {
-    my ( $slr, $lexer_name ) = @_;
-    my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-    my $lexer_id = $slg->[Marpa::R2::Internal::Scanless::G::LEXER_BY_NAME]->{$lexer_name};
-    Marpa::R2::exception( "Attempt to switch to unknown lexer: $lexer_name" ) if not defined $lexer_id;
-    my $thin_slr = $slr->[Marpa::R2::Internal::Scanless::R::C];
-    return $thin_slr->lexer_set($lexer_id);
-}
-
 my $libmarpa_trace_event_handlers = {
 
     'g1 accepted lexeme' => sub {
         my ( $slr, $event ) = @_;
-        my ( undef, undef, $lexeme_start_pos, $lexeme_end_pos, $g1_lexeme,
-            $lexer_id )
+        my ( undef, undef, $lexeme_start_pos, $lexeme_end_pos, $g1_lexeme)
             = @{$event};
         my $thin_slr = $slr->[Marpa::R2::Internal::Scanless::R::C];
         my $raw_token_value =
@@ -512,10 +599,7 @@ my $libmarpa_trace_event_handlers = {
             $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
         my $thick_g1_grammar = $thick_g1_recce->grammar();
         my $slg              = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
-        say {$trace_file_handle} qq{Lexer "$lexer_name" accepted lexeme },
+        say {$trace_file_handle} qq{Accepted lexeme },
             input_range_describe( $slr, $lexeme_start_pos,
             $lexeme_end_pos - 1 ),
             q{ e}, $slr->current_g1_location(),
@@ -528,8 +612,7 @@ my $libmarpa_trace_event_handlers = {
         my ( $slr, $event ) = @_;
         # Necessary to check, because this one can be returned when not tracing
         return if not $slr->[Marpa::R2::Internal::Scanless::R::TRACE_TERMINALS];
-        my ( undef, undef, $lexeme_start_pos, $lexeme_end_pos, $g1_lexeme,
-            $lexer_id )
+        my ( undef, undef, $lexeme_start_pos, $lexeme_end_pos, $g1_lexeme)
             = @{$event};
         my $thin_slr = $slr->[Marpa::R2::Internal::Scanless::R::C];
         my $raw_token_value =
@@ -541,10 +624,7 @@ my $libmarpa_trace_event_handlers = {
             $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
         my $thick_g1_grammar = $thick_g1_recce->grammar();
         my $slg              = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
-        say {$trace_file_handle} qq{Lexer "$lexer_name" rejected lexeme },
+        say {$trace_file_handle} qq{Rejected lexeme },
             input_range_describe( $slr, $lexeme_start_pos,
             $lexeme_end_pos - 1 ),
             q{: },
@@ -556,8 +636,7 @@ my $libmarpa_trace_event_handlers = {
         my ( $slr, $event ) = @_;
         # Necessary to check, because this one can be returned when not tracing
         return if not $slr->[Marpa::R2::Internal::Scanless::R::TRACE_TERMINALS];
-        my ( undef, undef, $position, $g1_lexeme, $assertion_id,
-            $lexer_id )
+        my ( undef, undef, $position, $g1_lexeme, $assertion_id)
             = @{$event};
         my ( $line, $column ) = $slr->line_column($position);
         my $thin_slr = $slr->[Marpa::R2::Internal::Scanless::R::C];
@@ -567,10 +646,7 @@ my $libmarpa_trace_event_handlers = {
             $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
         my $thick_g1_grammar = $thick_g1_recce->grammar();
         my $slg              = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
-        say {$trace_file_handle} qq{Lexer "$lexer_name" expected lexeme },
+        say {$trace_file_handle} qq{Expected lexeme },
             $thick_g1_grammar->symbol_in_display_form($g1_lexeme),
             " at line $line, column $column; assertion ID = $assertion_id"
             or Marpa::R2::exception("Could not say(): $ERRNO");
@@ -578,7 +654,7 @@ my $libmarpa_trace_event_handlers = {
     'outprioritized lexeme' => sub {
         my ( $slr, $event ) = @_;
         my ( undef, undef, $lexeme_start_pos, $lexeme_end_pos, $g1_lexeme,
-            $lexer_id, $lexeme_priority, $required_priority )
+            $lexeme_priority, $required_priority )
             = @{$event};
         my $thin_slr = $slr->[Marpa::R2::Internal::Scanless::R::C];
         my $raw_token_value =
@@ -590,11 +666,8 @@ my $libmarpa_trace_event_handlers = {
             $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
         my $thick_g1_grammar = $thick_g1_recce->grammar();
         my $slg              = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
         say {$trace_file_handle}
-            qq{Lexer "$lexer_name" outprioritized lexeme },
+            qq{Outprioritized lexeme },
             input_range_describe( $slr, $lexeme_start_pos,
             $lexeme_end_pos - 1 ),
             q{: },
@@ -650,7 +723,7 @@ my $libmarpa_trace_event_handlers = {
     },
     'lexer reading codepoint' => sub {
         my ( $slr, $event ) = @_;
-        my ( undef, undef, $codepoint, $position, $lexer_id ) = @{$event};
+        my ( undef, undef, $codepoint, $position, ) = @{$event};
         my $char      = chr $codepoint;
         my @char_desc = ();
         push @char_desc, qq{"$char"}
@@ -661,16 +734,13 @@ my $libmarpa_trace_event_handlers = {
         my $trace_file_handle =
             $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
         my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
         say {$trace_file_handle}
-            qq{Lexer "$lexer_name" reading codepoint $char_desc at line $line, column $column}
+            qq{Reading codepoint $char_desc at line $line, column $column}
             or Marpa::R2::exception("Could not say(): $ERRNO");
     },
     'lexer accepted codepoint' => sub {
         my ( $slr, $event ) = @_;
-        my ( undef, undef, $codepoint, $position, $token_id, $lexer_id ) =
+        my ( undef, undef, $codepoint, $position, $token_id ) =
             @{$event};
         my $char      = chr $codepoint;
         my @char_desc = ();
@@ -681,23 +751,20 @@ my $libmarpa_trace_event_handlers = {
         my $grammar = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
         my $thick_lex_grammar =
             $grammar->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]
-            ->[$lexer_id];
+            ->[0];
         my $symbol_in_display_form =
             $thick_lex_grammar->symbol_in_display_form($token_id),
             my ( $line, $column ) = $slr->line_column($position);
         my $trace_file_handle =
             $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
         my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
         say {$trace_file_handle}
-            qq{Lexer "$lexer_name" codepoint $char_desc accepted as $symbol_in_display_form at line $line, column $column}
+            qq{Codepoint $char_desc accepted as $symbol_in_display_form at line $line, column $column}
             or Marpa::R2::exception("Could not say(): $ERRNO");
     },
     'lexer rejected codepoint' => sub {
         my ( $slr, $event ) = @_;
-        my ( undef, undef, $codepoint, $position, $token_id, $lexer_id ) =
+        my ( undef, undef, $codepoint, $position, $token_id ) =
             @{$event};
         my $char      = chr $codepoint;
         my @char_desc = ();
@@ -708,60 +775,36 @@ my $libmarpa_trace_event_handlers = {
         my $grammar = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
         my $thick_lex_grammar =
             $grammar->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]
-            ->[$lexer_id];
+            ->[0];
         my $symbol_in_display_form =
             $thick_lex_grammar->symbol_in_display_form($token_id),
             my ( $line, $column ) = $slr->line_column($position);
         my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
         my $trace_file_handle =
             $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
         say {$trace_file_handle}
-            qq{Lexer "$lexer_name" codepoint $char_desc rejected as $symbol_in_display_form at line $line, column $column}
+            qq{Codepoint $char_desc rejected as $symbol_in_display_form at line $line, column $column}
             or Marpa::R2::exception("Could not say(): $ERRNO");
     },
     'lexer restarted recognizer' => sub {
         my ( $slr, $event ) = @_;
-        my ( undef, undef, $position, $lexer_id ) = @{$event};
+        my ( undef, undef, $position ) = @{$event};
         my ( $line, $column ) = $slr->line_column($position);
         my $trace_file_handle =
             $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
         my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
         say {$trace_file_handle}
-            qq{Lexer "$lexer_name" restarted recognizer at line $line, column $column}
-            or Marpa::R2::exception("Could not say(): $ERRNO");
-    },
-    'changing lexers' => sub {
-        my ( $slr, $event ) = @_;
-        my ( undef, undef, $position, $old_lexer_id, $new_lexer_id ) =
-            @{$event};
-        my ( $line, $column ) = $slr->line_column($position);
-        my $trace_file_handle =
-            $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
-        my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $old_lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$old_lexer_id];
-        my $new_lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$new_lexer_id];
-        say {$trace_file_handle}
-            qq{Changing lexers from "$old_lexer_name" to "$new_lexer_name" at line $line, column $column}
+            qq{Restarted recognizer at line $line, column $column}
             or Marpa::R2::exception("Could not say(): $ERRNO");
     },
     'discarded lexeme' => sub {
         my ( $slr, $event ) = @_;
-        my ( undef, undef, $lex_rule_id, $start, $end, $lexer_id ) =
+        my ( undef, undef, $lex_rule_id, $start, $end ) =
             @{$event};
         my $grammar = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
         my $thick_lex_grammar =
             $grammar->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]
-            ->[$lexer_id];
+            ->[0];
         my $grammar_c = $thick_lex_grammar->[Marpa::R2::Internal::Grammar::C];
         my $rule_length = $grammar_c->rule_length($lex_rule_id);
         my @rhs_ids =
@@ -770,12 +813,9 @@ my $libmarpa_trace_event_handlers = {
         my @rhs =
             map { $thick_lex_grammar->symbol_in_display_form($_) } @rhs_ids;
         my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
         my $trace_file_handle =
             $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
-        say {$trace_file_handle} qq{Lexer "$lexer_name" discarded lexeme },
+        say {$trace_file_handle} qq{Discarded lexeme },
             input_range_describe( $slr, $start, $end - 1 ), q{: }, join q{ },
             @rhs
             or Marpa::R2::exception("Could not say(): $ERRNO");
@@ -887,6 +927,19 @@ my $libmarpa_event_handlers = {
         return 1;
     },
 
+    'discarded lexeme' => sub {
+        my ( $slr,  $event )     = @_;
+        my ( undef, $rule_id, @other_data) = @{$event};
+        my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
+        my $lexeme_event =
+            $slg->[Marpa::R2::Internal::Scanless::G::DISCARD_EVENT_BY_LEXER_RULE]
+            ->[$rule_id];
+        push @{ $slr->[Marpa::R2::Internal::Scanless::R::EVENTS] },
+            [$lexeme_event, @other_data]
+            if defined $lexeme_event;
+        return 1;
+    },
+
     'unknown g1 event' => sub {
         my ( $slr, $event ) = @_;
         Marpa::R2::exception( ( join q{ }, 'Unknown event:', @{$event} ) );
@@ -945,9 +998,8 @@ sub Marpa::R2::Scanless::R::resume {
             my $stream_pos = $thin_slr->pos();
             my $trace_file_handle =
                 $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
-            my $lexer_id     = $thin_slr->current_lexer();
             my $thick_lex_grammar =
-                $slg->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]->[$lexer_id];
+                $slg->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]->[0];
             my $lex_tracer = $thick_lex_grammar->tracer();
             my ( $line, $column ) = $slr->line_column($stream_pos);
             print {$trace_file_handle}
@@ -988,12 +1040,8 @@ sub Marpa::R2::Scanless::R::resume {
 
         if ( $problem_code eq 'invalid char' ) {
             my $codepoint = $thin_slr->codepoint();
-            my $lexer_id  = $thin_slr->current_lexer();
-            my $lexer_name =
-                $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-                ->[$lexer_id];
             Marpa::R2::exception(
-                qq{Lexer "$lexer_name" failed at unacceptable character },
+                qq{Failed at unacceptable character },
                 character_describe( chr $codepoint ) );
         } ## end if ( $problem_code eq 'invalid char' )
 
@@ -1007,10 +1055,9 @@ sub Marpa::R2::Scanless::R::resume {
             # Recover by registering character, if we can
             my $codepoint = $thin_slr->codepoint();
             my $character = chr($codepoint);
-            my $lexer_id  = $thin_slr->current_lexer();
             my $character_class_table =
                 $slg->[Marpa::R2::Internal::Scanless::G::CHARACTER_CLASS_TABLES]
-                ->[$lexer_id];
+                ->[0];
             my @ops;
             for my $entry ( @{$character_class_table} ) {
 
@@ -1021,18 +1068,15 @@ sub Marpa::R2::Scanless::R::resume {
                         my $thick_lex_grammar =
                             $slg->[
                             Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]
-                            ->[$lexer_id];
+                            ->[0];
                         my $trace_file_handle = $slr->[
                             Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
                         my $char_desc = sprintf 'U+%04x', $codepoint;
                         if ( $character =~ m/[[:graph:]]+/ ) {
                             $char_desc .= qq{ '$character'};
                         }
-                        my $lexer_name =
-                            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-                            ->[$lexer_id];
                         say {$trace_file_handle}
-                            qq{Lexer "$lexer_name" registering character $char_desc as symbol $symbol_id: },
+                            qq{Registering character $char_desc as symbol $symbol_id: },
                             $thick_lex_grammar->symbol_in_display_form(
                             $symbol_id)
                             or
@@ -1078,9 +1122,8 @@ sub Marpa::R2::Scanless::R::read_problem {
     my $thin_slr = $slr->[Marpa::R2::Internal::Scanless::R::C];
     my $grammar  = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
 
-    my $lexer_id = $thin_slr->current_lexer();
     my $thick_lex_grammar =
-        $grammar->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]->[$lexer_id];
+        $grammar->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]->[0];
     my $lex_tracer = $thick_lex_grammar->tracer();
 
     my $trace_file_handle =
@@ -1129,7 +1172,7 @@ sub Marpa::R2::Scanless::R::read_problem {
             } ## end if ( scalar @events > 100 )
             EVENT: for my $event (@events) {
                 my ( $event_type, $trace_event_type, $lexeme_start_pos,
-                    $lexeme_end_pos, $g1_lexeme, $lexer_id )
+                    $lexeme_end_pos, $g1_lexeme )
                     = @{$event};
                 next EVENT
                     if $event_type ne q{'trace}
@@ -1145,9 +1188,6 @@ sub Marpa::R2::Scanless::R::read_problem {
                     $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
                 my $thick_g1_grammar = $thick_g1_recce->grammar();
                 my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-                $lexer_name =
-                    $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-                    ->[$lexer_id];
 
                 # Different internal symbols may have the same external "display form",
                 # which in naive reporting logic would result in many identical messages,
@@ -1155,9 +1195,8 @@ sub Marpa::R2::Scanless::R::read_problem {
                 # reports are not repeated, even when they have different causes
                 # internally.
 
-                $rejections{ qq{Lexer "}
-                        . $lexer_name . q{"; }
-                        . $thick_g1_grammar->symbol_in_display_form(
+                $rejections{ 
+                        $thick_g1_grammar->symbol_in_display_form(
                         $g1_lexeme)
                         . qq{; value="$raw_token_value"; length = }
                         . ( $lexeme_end_pos - $lexeme_start_pos ) } = 1;
@@ -1279,16 +1318,12 @@ sub Marpa::R2::Scanless::R::read_problem {
         my $trace_file_handle =
             $slr->[Marpa::R2::Internal::Scanless::R::TRACE_FILE_HANDLE];
         my $slg = $slr->[Marpa::R2::Internal::Scanless::R::GRAMMAR];
-        my $lexer_id = $thin_slr->current_lexer();
         my $thick_lex_grammar =
-            $grammar->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]->[$lexer_id];
+            $grammar->[Marpa::R2::Internal::Scanless::G::THICK_LEX_GRAMMARS]->[0];
         my $lex_tracer = $thick_lex_grammar->tracer();
-        my $lexer_name =
-            $slg->[Marpa::R2::Internal::Scanless::G::LEXER_NAME_BY_ID]
-            ->[$lexer_id];
         my ( $line, $column ) = $slr->line_column($stream_pos);
         $read_string_error .=
-            qq{\n=== Progress report for lexer "$lexer_name" at line $line, column $column\n} .
+            qq{\n=== Progress report for lexer at line $line, column $column\n} .
             $lex_tracer->lexer_progress_report($slr);
     }
 
@@ -1402,14 +1437,17 @@ sub Marpa::R2::Scanless::R::ambiguity_metric {
     my $thick_g1_recce =
         $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
     my $ordering = $thick_g1_recce->ordering_get();
-    return 0 if not $ordering;
-    return $ordering->ambiguity_metric();
+    my $metric = $ordering ? $ordering->ambiguity_metric() : 0;
+    my $bocage = $thick_g1_recce->[Marpa::R2::Internal::Recognizer::B_C];
+    return $metric;
 } ## end sub Marpa::R2::Scanless::R::ambiguity_metric
 
 sub Marpa::R2::Scanless::R::ambiguous {
     my ($slr) = @_;
-    return q{No parse} if $slr->ambiguity_metric() <= 0;
-    return q{} if $slr->ambiguity_metric() == 1;
+    local $Marpa::R2::Context::slr = $slr;
+    my $ambiguity_metric = $slr->ambiguity_metric();
+    return q{No parse} if $ambiguity_metric <= 0;
+    return q{} if $ambiguity_metric == 1;
     my $asf = Marpa::R2::ASF->new( { slr => $slr } );
     die 'Could not create ASF' if not defined $asf;
     my $ambiguities = Marpa::R2::Internal::ASF::ambiguities($asf);
@@ -1533,7 +1571,7 @@ sub Marpa::R2::Scanless::R::series_restart {
     $slr->[Marpa::R2::Internal::Scanless::R::REJECTION_ACTION] = 'fatal';
 
     $thick_g1_recce->reset_evaluation();
-    my $g1_recce_args = Marpa::R2::Internal::Scanless::R::set($slr, "series_restart", @args );
+    my ($g1_recce_args) = Marpa::R2::Internal::Scanless::R::set($slr, "series_restart", @args );
     $thick_g1_recce->set( $g1_recce_args );
     return 1;
 }
