@@ -1,4 +1,4 @@
-# Copyright 2014 Jeffrey Kegler
+# Copyright 2015 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -20,7 +20,7 @@ use warnings;
 use strict;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.102000';
+$VERSION        = '2.104000';
 $STRING_VERSION = $VERSION;
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -367,7 +367,7 @@ sub code_problems {
 
 } ## end sub code_problems
 
-# Dump semnatics for diagnostics
+# Dump semantics for diagnostics
 sub show_semantics {
     my (@ops)    = @_;
     my @op_descs = ();
@@ -957,13 +957,15 @@ sub registration_init {
 
     my $null_values = $recce->[Marpa::R2::Internal::Recognizer::NULL_VALUES];
 
-    state $op_bless         = Marpa::R2::Thin::op('bless');
-    state $op_callback      = Marpa::R2::Thin::op('callback');
-    state $op_push_constant = Marpa::R2::Thin::op('push_constant');
-    state $op_push_length   = Marpa::R2::Thin::op('push_length');
-    state $op_push_undef    = Marpa::R2::Thin::op('push_undef');
-    state $op_push_one      = Marpa::R2::Thin::op('push_one');
-    state $op_push_sequence = Marpa::R2::Thin::op('push_sequence');
+    state $op_bless          = Marpa::R2::Thin::op('bless');
+    state $op_callback       = Marpa::R2::Thin::op('callback');
+    state $op_push_constant  = Marpa::R2::Thin::op('push_constant');
+    state $op_push_g1_length = Marpa::R2::Thin::op('push_g1_length');
+    state $op_push_length    = Marpa::R2::Thin::op('push_length');
+    state $op_push_undef     = Marpa::R2::Thin::op('push_undef');
+    state $op_push_one       = Marpa::R2::Thin::op('push_one');
+    state $op_push_sequence  = Marpa::R2::Thin::op('push_sequence');
+    state $op_push_g1_start  = Marpa::R2::Thin::op('push_g1_start');
     state $op_push_start_location =
         Marpa::R2::Thin::op('push_start_location');
     state $op_push_values        = Marpa::R2::Thin::op('push_values');
@@ -981,7 +983,7 @@ sub registration_init {
         my $semantic_rule = $null_values->[$nulling_symbol];
         next NULLING_SYMBOL if not defined $semantic_rule;
         $nulling_symbol_by_semantic_rule[$semantic_rule] = $nulling_symbol;
-    }
+    } ## end NULLING_SYMBOL: for my $nulling_symbol ( 0 .. $#{$null_values} )
 
     my @work_list = ();
     RULE: for my $rule_id ( $grammar->rule_ids() ) {
@@ -1013,6 +1015,7 @@ sub registration_init {
     # it may be best to have a separate semantics object.
     my @nulling_closures = ();
     my @registrations    = ();
+    my $top_nulling_ops;
 
     WORK_ITEM: for my $work_item (@work_list) {
         my ( $rule_id, $lexeme_id, $semantics, $blessing ) = @{$work_item};
@@ -1021,10 +1024,10 @@ sub registration_init {
             $is_discard_sequence_rule, $nulling_symbol_id );
         if ( defined $rule_id ) {
             $nulling_symbol_id = $nulling_symbol_by_semantic_rule[$rule_id];
-            $closure           = $closure_by_rule_id[$rule_id];
-            $rule              = $rules->[$rule_id];
-            $rule_length       = $grammar_c->rule_length($rule_id);
-            $is_sequence_rule  = defined $grammar_c->sequence_min($rule_id);
+            $closure          = $closure_by_rule_id[$rule_id];
+            $rule             = $rules->[$rule_id];
+            $rule_length      = $grammar_c->rule_length($rule_id);
+            $is_sequence_rule = defined $grammar_c->sequence_min($rule_id);
             $is_discard_sequence_rule = $is_sequence_rule
                 && $rule->[Marpa::R2::Internal::Rule::DISCARD_SEPARATION];
         } ## end if ( defined $rule_id )
@@ -1179,6 +1182,14 @@ sub registration_init {
             for my $result_descriptor ( split /[,]\s*/xms, $array_descriptor )
             {
                 $result_descriptor =~ s/^\s*|\s*$//g;
+                if ( $result_descriptor eq 'g1start' ) {
+                    push @push_ops, $op_push_g1_start;
+                    next RESULT_DESCRIPTOR;
+                }
+                if ( $result_descriptor eq 'g1length' ) {
+                    push @push_ops, $op_push_g1_length;
+                    next RESULT_DESCRIPTOR;
+                }
                 if ( $result_descriptor eq 'start' ) {
                     push @push_ops, $op_push_start_location;
                     next RESULT_DESCRIPTOR;
@@ -1289,21 +1300,6 @@ sub registration_init {
 
         if ( defined $nulling_symbol_id ) {
 
-            my $slr = $Marpa::R2::Context::slr;
-            if (    defined $slr
-                and $tracer->symbol_name($nulling_symbol_id) eq '[:start]'
-                and defined(
-                    my $default_g1_start_closure =
-                        $slr->default_g1_start_closure()
-                )
-                )
-            {
-                # Special case for SLIF nulling start symbol when there is a default action
-                $nulling_closures[$nulling_symbol_id] =
-                    $default_g1_start_closure;
-                @ops = ($op_callback);
-            } ## end if ( defined $slr and $tracer->symbol_name(...))
-
             push @registrations, [ 'nulling', $nulling_symbol_id, @ops ];
         } ## end if ( defined $nulling_symbol_id )
 
@@ -1311,14 +1307,49 @@ sub registration_init {
             push @registrations, [ 'token', $lexeme_id, @ops ];
         }
 
-        $recce->[Marpa::R2::Internal::Recognizer::REGISTRATIONS] =
-            \@registrations;
-        $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_SYMBOL_ID] =
-            \@nulling_closures;
-        $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_RULE_ID] =
-            \@closure_by_rule_id;
-
     } ## end WORK_ITEM: for my $work_item (@work_list)
+
+    SLR_NULLING_GRAMMAR_HACK: {
+        last SLR_NULLING_GRAMMAR_HACK if not $Marpa::R2::Context::slr;
+
+        # A hack for nulling SLR grammars --
+        # the nulling semantics of the start symbol should
+        # be those of the symbol on the
+        # RHS of the start rule --
+        # so copy them.
+
+        my $start_symbol_id = $tracer->symbol_by_name('[:start]');
+        last SLR_NULLING_GRAMMAR_HACK
+            if not $grammar_c->symbol_is_nullable($start_symbol_id);
+
+        my $start_rhs_symbol_id;
+        RULE: for my $rule_id ( $grammar->rule_ids() ) {
+            my ( $lhs, $rhs0 ) = $tracer->rule_expand($rule_id);
+            if ( $start_symbol_id == $lhs ) {
+                $start_rhs_symbol_id = $rhs0;
+                last RULE;
+            }
+        } ## end RULE: for my $rule_id ( $grammar->rule_ids() )
+
+        REGISTRATION: for my $registration (@registrations) {
+            my ( $type, $nulling_symbol_id ) = @{$registration};
+            if ( $nulling_symbol_id == $start_rhs_symbol_id ) {
+                my ( undef, undef, @ops ) = @{$registration};
+                push @registrations, [ 'nulling', $start_symbol_id, @ops ];
+                $nulling_closures[$start_symbol_id] =
+                    $nulling_closures[$start_rhs_symbol_id];
+                last REGISTRATION;
+            } ## end if ( $nulling_symbol_id == $start_rhs_symbol_id )
+        } ## end REGISTRATION: for my $registration (@registrations)
+    } ## end SLR_NULLING_GRAMMAR_HACK:
+
+    $recce->[Marpa::R2::Internal::Recognizer::REGISTRATIONS] =
+        \@registrations;
+    $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_SYMBOL_ID] =
+        \@nulling_closures;
+    $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_RULE_ID] =
+        \@closure_by_rule_id;
+
 } ## end sub registration_init
 
 # Returns false if no parse
@@ -1560,13 +1591,24 @@ sub Marpa::R2::Recognizer::value {
     {
         my ( $type, $id, @raw_ops ) = @{$registration};
         my @ops = ();
-        if ( $trace_values > 2 ) {
+        PRINT_TRACES: {
+            last PRINT_TRACES if $trace_values <= 2;
+            if ( $type eq 'nulling' ) {
+                say {$trace_file_handle}
+                    "Registering semantics for nulling symbol: ",
+                    $grammar->symbol_name($id),
+                    "\n", '  Semantics are ', show_semantics(@raw_ops)
+                    or
+                    Marpa::R2::exception('Cannot say to trace file handle');
+                last PRINT_TRACES;
+            } ## end if ( $type eq 'nulling' )
             say {$trace_file_handle}
                 "Registering semantics for $type: ",
                 $grammar->symbol_name($id),
                 "\n", '  Semantics are ', show_semantics(@raw_ops)
                 or Marpa::R2::exception('Cannot say to trace file handle');
-        } ## end if ( $trace_values > 2 )
+        } ## end PRINT_TRACES:
+
         OP: for my $raw_op (@raw_ops) {
             if ( ref $raw_op ) {
                 push @ops, $value->constant_register( ${$raw_op} );
